#!/bin/sh

# NAME:
#	picalbum - prepare pics for display on the web
#
# SYNOPSIS:
#	picalbum [options]
#
# DESCRIPTION:
#	For all pics (*.jpg *.tif *.crw ...) generate a set of
#	connected "slides" to display them.
#	By default we decend into subdirectories - thus one invocation
#	can take care of a whole album.
#
#	We actually gather all the file names of the pics and strip
#	the extensions to get a sorted list of the basenames that we
#	will hook together as slides.  This makes it easier to deal
#	with things like Canon RAW images (.crw) which we don't
#	display directly.
#
#	For each "PIC" ("file"."ext"), we aim to have a thumbnail
#	"file"-t.jpg (refered to as THM) and a scaled web image
#	"file"-w.jpg. (refered to as WEB).
#	If the original pic is small enough (and does not need
#	rotation) we will skip the scaled image.
#
#	I _never_ touch the original image off the camera, so if a pic
#	needs post processing, I'll save the original as
#	raw/"pic" and work on a copy.  Thus if raw/"pic" exists a link
#	to it will be put in the slide ("file".html).
#	Similarly if "file".tif, "file".crw or "file".cr2 exists
#	we'll add links to those.
#
#	Also, because I never modify the original pics, I record
#	rotation data in a separate file, and this is used to rotate
#	the THM and WEB images.
#
#	Since the pics from an 8Mpixel camera are large, we try to
#	generate the THM from the WEB image when possible.  This can
#	significantly speed up processing.
#
#	I use mipcs.pl(1) to rename the original pics to
#	"YYYYmmddNNN".* and  store them in a subdir like "YYYY/mm".
#	After copying in new pics, I used to:
#.nf
#
#		picalbum --noWeb [YYYY/mm [...]]
#.fi
#
#	To generate just thumbnails and index, then look at those and
#	run mkrotate(1) in each subdir if needed to record rotation data,
#	then
#.nf
#
#		picalbum  [YYYY/mm [...]]
#.fi
#
#	To generate the scaled images and slides (linked .html pages)
#	and regenerate the thumbnails if needed.  Finally
#.nf
#
#		picalbum  --noSlides
#.fi
#
#	To regenerate all the index files so that the entire album is
#	linked together correctly.
#
#	These days I use pictool.py(1) which is a simple Python script,
#	to display the new images one at a time and record the rotate
#	data, discard junk and select images for posting in the public
#	galleries.
#	
#	For Canon RAW images (.crw), we either extract the embeded
#	jpeg (if there is one) using exiftool or convert the crw image
#	to a ppm image, and do further processing with the jpeg or ppm image.
#	The selection is made by setting CRW2EXT to jpg or ppm
#	and setting EXIFTOOL or DCRAW to point to the appropriate binary.
#	I prefer to shoot RAW+JPEG which the 20D stores in separate
#	files, rather than just RAW.
#
#	Text:
#
#	Since all the html files in an album are generated, we use
#	.txt files to store text that should be included.
#	
#	When generating an index we look for index-head.txt and
#	index-tail.txt to include before/after the image thumnails.
#
#	When generating slides, we look for "file".exif1 which should
#	contain a summary of the EXIF data (generated by another
#	Python script exif.py(1)).  If "file".txt exists it is appended to the
#	slide after the image and EXIF summary. 
#	
#	Configuration:
#
#	Most configuration is static and handled by setting variables
#	(listed below) in a $rc file (.picalbumrc), located in $HOME
#	and various directories within the album.  The main reason for
#	a $rc in a subdirectory of an album is to cause some subdirs
#	to be excluded from processing.  Each directory within the
#	album can also contain a meta.properties file to control the
#	title, description and folderIcon used for that directory.
#
#	The variables one can set in $rc (.picalbumrc) are:
#	
#	JPEG_QUALITY
#		JPEG is a lossy compression scheme, using a high
#		"quality" setting (94) reduces the impact.
#
#	WEB_GEOMETRY
#		The dimension limits (760x700) of the scaled images
#		shown in the slides.
#
#	THM_GEOMETRY
#		The dimension limits (100x100) of the thumbnails.
#
#	CRW2EXT	The extension used for the image we convert Canon CRW
#		images to (jpg).  The choice depends on the tool being
#		used (either EXIFTOOL or DCRAW).
#
#	ORIGINAL_IMAGES
#		As mentioned, I never touch the original images from
#		the camera.  If they need post processing, the
#		original is copied to $ORIGINAL_IMAGES (raw), and we
#		will include a link to it in the slide.
#
#	PRINT_IMAGES
#		Most digital images need serious sharpening before
#		printing.   This variable names a directory (print)
#		where the sharpened images are collected.  I do this
#		in batch but only for the ones I plan to print.
#
#	PRINT_IMAGE_SUBDIRS
#		If you prepare prints for multiple printers, put them
#		in a named subdir of $PRINT_IMAGES and add that subdir
#		to this.
#
#	ROTATE_LIST
#		The file used to record rotation data (.rotate).
#
#	ICONS	Where icons are to be found (/icons).
#
#	FOLDER_ICON
#		Default folderIcon ($ICONS/folder.gif).
#		
#	HTML_EXT
#		The extension used for HTML files (.htm).
#
#	INDEX_NAME
#		The name used for index files (index$HTML_EXT).
#
#	STYLESHEET_DIR
#		The place to look for the stylesheet (res).
#		This is typically a subdir at the top of the album,
#		and used to help find the top level of the album (see
#		TOP_CLUE). 
#
#	STYLESHEET
#		The stylesheet to use (styles.css).
#
#	UNSHARP
#		Apply unsharp mask when generating WEB and THM images
#		(1x50).
#
#	TOP_CLUE
#		What to look for when searching for the top of the
#		album ($STYLESHEET_DIR/$STYLESHEET).
#
#	COLS	How many thumbnails per row in the index (7).
#
#
#	PICS
#		List of extensions that indicate an image we should
#		process (.jpg .crw .tif .gif .ppm .pnm).
#		
#	Options:
#
#	
#	--noWeb
#		Skip generating scaled images - since we expect to
#		have to re-do it after running mkrotate(1).
#
#	--noSubirs
#		Do not process any subdirs.
#
#	--noSlides
#		Just regenerate the index files.
#
#	--noIndex
#		Just regenerate the slides.
#
#	--skipDir "dir"
#		Add "dir" to the list of subdirs to exclude.
#		You can use exclude_subdir "dir" in $rc too.
#
#	--noUnsharp
#		Skip the unsharp
#
#	--Unsharp "<radius>x<sigma>"
#		Use the specified radius and deviation.
#
# FILES:
#	\&.picalbumrc	rc file to control default behavior.
#
#	meta.properties	description of a subdir within album.
#
#	index-head.txt	text to include in index.htm above image
#			thumbnails. 
#
#	index-tail.txt	text to include in index.htm after image
#			thumbnails. 
#
#	"file".exif1	file containing summary of EXIF data.
#
#	"file".ref	generated file, contains hyperlink to slide.
#			This is incorporated into the index page.
#
#	"file".txt	text to include in the slide after the image
#			and EXIF data.
#
# SEE ALSO:
#	mpics.pl(1), photoweb(1), pictool.py(1), exif.py(1),
#	ImageMagik, dcraw, exiftool.
#	
# AUTHOR:
#	Simon J. Gerraty <sjg@crufty.net>

# RCSid:
#	$Id: picalbum.sh,v 1.40 2005/07/19 00:57:35 sjg Exp $
#
#	@(#)Copyright (c) 2003-2004 Simon J. Gerraty
#	
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to use, modify and distribute this source code 
#	is granted subject to the following conditions.
#	1/ that the above copyright notice and this notice 
#	are preserved in all copies and that due credit be given 
#	to the author.	
#
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

case `echo -n .` in -n*) _N=; _C="\c";; *) _N="-n"; _C=;; esac

Myname=${Myname:-`basename $0 .sh`}
Mydir=`dirname $0`
case $Mydir in
/*) ;;
*) Mydir=`cd $Mydir; pwd`;;
esac
PATH=$PATH:$Mydir

rc=.${Myname}rc

# add . debug.sh to $rc if you
# have it and want to debug things.
DebugOn() {
        :
}

DebugOff() {
        :
}

noIndex=
noSubdirs=
noSlides=
# this has no effect unless ImageBorder is also enabled.
noImageCopyright=
noImageBorder=:
# even if ImageBorder is true, we don't want borders on thm's
doThmBorder=

RAW_PICS=".crw .cr2"
PICS=".jpg .jpeg .tif .tiff .gif .ppm .pnm .pgm .bmp"
EXCLUDE_SUBDIRS=
# a little unsharp makes a big difference
UNSHARP=1x50

PRINT_IMAGES=print
PRINT_PICS=".jpg .tif"

exclude_subdir() {
	EXCLUDE_SUBDIRS="$1${EXCLUDE_SUBDIRS:+|$EXCLUDE_SUBDIRS}"
}

exists() {
	case "$1" in
	-?) _t=$1; shift;;
	*) _t=-s;;
	esac
	for f in $*
	do
		test $_t $f || continue
		echo $f
		break
	done
}


newer() {
	/bin/ls -1t $* 2>/dev/null | head -1
}

uptodate() {
        _newer=`newer $*`
	case $_newer in
	$1) return 0;;
	esac
	return 1
}

toUpper() {
	tr abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ
}

divClass() {
	echo "<DIV CLASS=\"$1\">"
	if [ -f $2 ]; then
		cat $2
	else
		shift
		echo "$@"
	fi
	echo '</DIV>'
}

divId() {
	echo "<DIV ID=\"$1\">"
	if [ -f $2 ]; then
		cat $2
	else
		shift
		echo "$@"
	fi
	echo '</DIV>'
}

NewExt=new
OldExt=old

install_if_new() {
	if test -s $1.$NewExt; then
		if test -f $1; then
			if cmp -s $1 $1.$NewExt; then
				rm -f $1.$NewExt
			else
				rm -f $1.$OldExt
				mv $1 $1.$OldExt 
			fi
		fi
	else
		rm -f $1.$NewExt
	fi
	
	if test -s $1.$NewExt; then
		mv $1.$NewExt $1 && { ${ECHO:-:} $1; return 0; }
	fi
	return 1
}

findTOP() {
	rtop=
	rcfiles=
	clue=${TOP_CLUE:-$STYLESHEET_DIR/$STYLESHEET}
	for p in .. `IFS=/; echo $curdir`
	do
		if [ -s $rtop$rc ]; then
			rcfiles="${rtop:-./}$rc $rcfiles"
		fi
		if [ -s $rtop$clue ]; then
			TOP=`cd ${rtop:-.} && pwd`
			return
		fi
		rtop=../$rtop
	done
}

findRTOP() {
	case $curdir/ in
	$TOP/)	RTOP=; RDIRS=;;
	*)
		RTOP="`echo $curdir | sed -e "s,$TOP/,," -e 's,[^/][^/]*,..,g'`/"
		RSUB=`echo $curdir | sed -e "s,$TOP/,,"`
		RDIRS=`IFS=/; echo $RSUB`
		;;
	esac
	if [ -s $RTOP$STYLESHEET_DIR/$STYLESHEET ]; then
		CSS=$RTOP$STYLESHEET_DIR/$STYLESHEET
	fi
	if [ -s $RTOP$STYLESHEET_DIR/$PRINT_STYLESHEET ]; then
		PRINT_CSS=$RTOP$STYLESHEET_DIR/$PRINT_STYLESHEET
	fi
}

# replace this in .rc if it doesn't work
ls_size() {
        /bin/ls -Ll $1 | (read m l u g b x; expr $b / 1024)
}


getDIM() {
        DebugOn getDIM getDIM:$1
        WIDTH=
        if [ -z "$noCacheDIM" ] && uptodate $2 $1; then
                case "$1" in
		*/*) ref=`basename $1`;;
		*) ref=$1;;
		esac
		DIM=`grep "[/\"]$ref.*WIDTH=" $2 | sed -e 's,.*\(WIDTH=[^=]*HEIGHT=[^ ]*\) .*,\1,' -e 's,",,g'`
                case "$DIM" in
		*=*) eval "$DIM";;
		esac
	fi
        if [ -z "$WIDTH" ]; then
                eval `$IDENTIFY $1 | sed 's,.* \([1-9][0-9]*\)x\([^ ]*\) .*,DIM=\1x\2 WIDTH=\1 HEIGHT=\2,'`
	else
                DIM=${WIDTH}x$HEIGHT
	fi
        DebugOff getDIM getDIM:$1
}

pic_info() {
	_pic=$1
	_prefix=$2
	
	case "$1" in
	"") return 1;;
	*.jp*) eval "${2}TYPE=JPEG";;
	*) eval "${2}TYPE=`expr $1 : '.*\.\([^.]*\)' | toUpper`";;
	esac
        _size=`du -s $du_k < $1 | cut -f1`
        if [ $_size -lt 1 ]; then
                _size=`ls_size $1`
	fi
	eval "${2}SIZE=$_size"
}

navlink() {
	name=$1
	link=$2

	case "$link" in
	"")	echo $name;;
	*)	echo "<A HREF=\"$link\">$name</A>";;
	esac
}

piclink() {
	if [ -s $1 ]; then
		pic_info $1 PLINK_
		navlink "$2${2:+ }$PLINK_TYPE Image [${PLINK_SIZE}KB]" $1
	fi
}

get_title() {
	echo $RSUB
}

get_year() {
	echo ${copyright_year:-$COPYRIGHT_YEAR}
}

get_image_copyright() {
	font= text= text_xy=
	case "$1" in
	web)	font=$WEB_TEXT_FONT
		text_xy=$WEB_IMAGE_COPYRIGHT_XY
		;;
	esac
	font=${font:-${IMAGE_TEXT_FONT:-6x9}}
	# the default xy places the text to the left of the image.
	# if you want it to the right, you need to compute
	# x=(font-width * strlen(text)) + border-width
	# y=(border-height - font-height) / 2
	# and use -$x,-$y
	text_xy=${text_xy:-${IMAGE_COPYRIGHT_XY:-10,-2}}
	text="${IMAGE_COPYRIGHT_TEXT:-Copyright (c) `get_year` ${IMAGE_COPYRIGHT_URL:-$FOOTER_NAME}}"
	$noImageCopyright echo "-font $font -pen ${IMAGE_TEXT_COLOR:-white} -draw 'text $text_xy \"$text\"'"
}

get_image_border() {
	border_xy=
	case "$1" in
	web) border_xy=$WEB_BORDER;;
	esac
	border_xy=${border_xy:-${BORDER_XY:-8x12}}
	$noImageBorder echo "-bordercolor ${BORDER_COLOR:-black} -border $border_xy"
}

navtab() {
	class=$1
	id=$2
	title=$3

	cat << EON
<DIV CLASS="noprint">
<TABLE WIDTH="100%">
<TBODY>
<TR CLASS="${class:-head}"><TD ALIGN="LEFT"><STRONG>
<A HREF="$RTOP$INDEX_NAME">${ALBUM_NAME:-Top}</A>
EON
	r=$RTOP
	for d in $RDIRS
	do
		r=`dirname $r`
		case "$r" in
		.) r=;;
		*) r=$r/;;
		esac
		echo "| <A HREF=\"${r}$INDEX_NAME\">$d</A>"
	done
	case "$id" in
	index)
		if [ "$prevdir$nextdir" ]; then
			cat << EON1
</STRONG></TD>
<TD CLASS="navigation" ALIGN="RIGHT">
<STRONG>
EON1
			navlink previous ${prevdir:+../$prevdir/$INDEX_NAME}
			echo "|"
			navlink next ${nextdir:+../$nextdir/$INDEX_NAME}
		fi
		;;
	slide)	cat << EON2
| $title
</STRONG></TD>
<TD CLASS="navigation" ALIGN="RIGHT">
<STRONG>
EON2

	if [ -d ${prev:-/dev/null} ]; then
                navlink PREVIOUS $prev/$INDEX_NAME
	else
        	navlink previous $prev${prev:+$HTML_EXT}
	fi
	echo "|"
	if [ -d ${next:-/dev/null} ]; then
        	navlink NEXT $next/$INDEX_NAME
	else
        	navlink next $next${next:+$HTML_EXT}
	fi
	;;
	esac
	cat << EON3
</STRONG></TD>
</TR>
</TBODY>
</TABLE>
</DIV>
EON3
}

htm_header() {
	id="$1"
	css="$2"
	title="$3"

	DebugOn htm_header htm_header:$id htm_header:$title
	cat << EOH
<HTML>
<HEAD>
<TITLE>$title</TITLE>
${css:+<LINK HREF=\"$css\" rel=\"stylesheet\" media=\"screen\"/>}
${css:+<LINK HREF=\"$PRINT_CSS\" rel=\"stylesheet\" media=\"print\"/>}
</HEAD>
<BODY ID="$id">
EOH
	navtab head $id "$title"
	DebugOff htm_header htm_header:$id htm_header:$title
}

htm_footer() {
	id=$1
	title="$2"

	DebugOn htm_footer htm_footer:$id htm_header:$title
	year=`get_year $*`
	navtab foot $id "$title"
                
	cat << EOF
<P CLASS="footer"><CENTER><SMALL>
${FOOTER_URL:+<A HREF=\"$FOOTER_URL\">$FOOTER_NAME</A><BR>}
${COPYRIGHT_URL:+<A HREF=\"$COPYRIGHT_URL\">}Copyright &#169; $year${COPYRIGHT_URL:+</A>}
</SMALL></CENTER>
</P>
</BODY>
</HTML>
EOF
	DebugOff htm_footer htm_footer:$id htm_header:$title
}

get_properties() {
	DebugOn get_properties get_properties:$1 get_properties:$curdir:$1
	if [ -s $1 ]; then
		eval `sed -n -e /^#/d -e "/${3:-.}/s,^\([^= ]*\) *= *\([^ ].*\),$2\1=\"\2\",p" $1`
	fi
	DebugOff get_properties get_properties:$1 get_properties:$curdir:$1
}

getIcon() {
	DebugOn getIcon getIcon:$1
	_folderIcon=
	_folderLabel=
	_title=
	_description=
	get_properties $1/meta.properties _ folderIcon
	icon=`exists -f $_folderIcon $1/$_folderIcon $1/*/$_folderIcon`
	if [ -z "$icon" ]; then
		get_properties `exists $1/*/meta.properties` _ folderIcon
		icon=`exists -f $_folderIcon $1/$_folderIcon $1/*/$_folderIcon`
	fi
	icon=${icon:-`exists -f $1/*-t.jpg $1/*/*-t.jpg`}
	icon=${icon:-$FOLDER_ICON}
	DebugOff getIcon getIcon:$1
}

doCRW() {
	case "$2" in
	*.jpg)
		if [ -x ${EXIFTOOL:-/dev/null} ]; then
			$EXIFTOOL -b -JpgFromRaw $1 > $2
		fi
		;;
	*.ppm)
		if [ -x ${DCRAW:-/dev/null} ]; then
			$DCRAW -c -w $1 > $2
		fi
		;;
	esac
}

doWEB() {
	DebugOn web $this:web
	web=${this}-w.jpg
	WEB=`exists $web`
	if [ $PIC_SIZE -gt ${opt_s:-80} -o "x$ROTATE" != x -o $PIC_TYPE = TIF ]; then
		WEB=$web
	else
		WEB=${WEB:-$PIC}
	fi
	if ! uptodate $WEB $PIC; then
	    DebugOn $PIC:convert $this:convert convert
	    echo "  Generating $WEB"
	    eval "$DOIT $CONVERT $ROTATE  ${JPEG_QUALITY:+-quality} $JPEG_QUALITY -geometry \"$WEB_GEOMETRY>\" `get_image_border web` `get_image_copyright web` ${UNSHARP:+-unsharp $UNSHARP} $PIC $WEB"
	    DebugOff $PIC:convert $this:convert convert
	fi
	pic_info $WEB WEB_
	DebugOff web $this:web
}


doTHM() {
	DebugOn thm $this:thm
	THM=${this}-t.jpg
	if ! uptodate $THM $PIC; then
		DebugOn $PIC:convert $this:convert convert
		echo "	Generating $THM"
		eval "$DOIT $CONVERT $ROTATE ${JPEG_QUALITY:+-quality} $JPEG_QUALITY -geometry $THM_GEOMETRY ${THM_UNSHARP:+-unsharp $THM_UNSHARP} ${1:-$PIC} $THM"
		DebugOff $PIC:convert $this:convert convert
	fi
	DebugOff thm $this:thm
}
        

doPIC() {
	DebugOn pic $this:pic
	ROTATE=
	pics=
	for e in $PICS
	do
		pics="$pics $this$e"
	done
	PIC=`exists $pics`
	case "$PIC" in
	"")
		for e in $RAW_PICS
		do
			if [ -s $this$e ]; then
				DebugOn crw
				PIC=$this.$CRW2EXT
				[ -s $PIC ] || doCRW $this$e $PIC
				DebugOff crw
			fi
		done
		if [ ! -s ${PIC:-/dev/null} ]; then
			echo Skipping $this
			DebugOff pic $this:pic
			return 1
		fi
		;;
	esac
	pic_info $PIC PIC_
	if [ -s $ROTATE_LIST ]; then
		DebugOn $PIC:rotate $this:rotate rotate
		ROTATE=`grep "^$this" $ROTATE_LIST | cut -d: -f2 | tail -1`
		if [ "x$ROTATE" != x ]; then
			ROTATE="-rotate $ROTATE"
		fi
	fi
	WEB=
	THM=
        THM_UNSHARP=$UNSHARP
	case "$WEB_GEOMETRY" in
	no*)	echo " Skipping $WEB etc."
		doTHM $PIC
		DebugOff pic $this:pic
		return 0
		;;
	esac
	doWEB
	DebugOn picthm $this:picthm
	# if we now have $WEB, it's much quicker to generate the thm from that
	# but we generally don't want borders on the thm's
	${noImageBorder:-$doThmBorder} doTHM $PIC
	if [ $WEB != $PIC ]; then
		ROTATE=
		THM_UNSHARP=
	fi
	doTHM $WEB
	DebugOff pic $this:pic picthm $this:picthm
	return 0
}

doSLIDE() {
	prev=$1
	this=$2
	next=$3
	css=$4

	DebugOn doPIC
	slide=$this$HTML_EXT
	SLIDE=$slide.$NewExt
	echo Processing $this...
	htm_header slide "$css" "$this" > $SLIDE
	[ -s $this.hdr ] && cat $this.hdr >> $SLIDE
	cat << EOH >> $SLIDE
<TABLE WIDTH=0 ALIGN="center" BORDER=0>
<TBODY><TR><TD COLSPAN=2>
EOH
	doPIC $this
	if ! uptodate ${this}-w.ref $WEB; then
		getDIM $WEB $slide
		case "$PIC" in
		$WEB|*.tif)
			echo "<IMG SRC=\"$WEB\" ${ALIGN:+ALIGN=\"$ALIGN\"} WIDTH=\"$WIDTH\" HEIGHT=\"$HEIGHT\" BORDER=0 ALT=\"$DIM ${WEB_SIZE}KB $WEB_TYPE\">" > ${this}-w.ref
			;;
		*)
			echo "<A HREF=\"$PIC\"><IMG SRC=\"$WEB\" ${ALIGN:+ALIGN=\"$ALIGN\"} WIDTH=\"$WIDTH\" HEIGHT=\"$HEIGHT\" BORDER=0 ALT=\"$DIM ${PIC_SIZE}KB $PIC_TYPE\"></A>" > ${this}-w.ref
			;;
		esac
	fi
	divClass slide-image ${this}-w.ref >> $SLIDE
	RAW=`piclink $ORIGINAL_IMAGES/$PIC Original`
	TIF=`piclink $this.tif`
	CRW=`piclink $this.crw Canon`
	CRW=${CRW:-`piclink $this.crw2 Canon`}
	PRINT=
	for d in $PRINT_IMAGES
	do
		for s in "" $PRINT_IMAGE_SUBDIRS
		do
		      for e in $PRINT_PICS
		      do
			      x=`piclink $d${s:+/$s}/$this$e "Print $s"`
			      if [ "$x" ]; then
				      eval "Print${s:+_$s}='$x'"
				      PRINT="$PRINT Print${s:+_$s}"
			      fi
		      done
		done
	done
	if [ "x$RAW$TIF$CRW$PRINT" != x ]; then
		echo "</TD></TR><TR><TD COLSPAN=2>"
		for x in RAW TIF CRW $PRINT
		do
			eval "echo \"\$$x\${$x:+<BR>}\""
		done
			
	fi >> $SLIDE
	cat << EOB >> $SLIDE
</TD></TR></TABLE>
<CENTER>
<TABLE WIDTH="$SLIDE_TEXT_WIDTH">
<TR><TD COLSPAN=2 ALIGN="center"><H3 CLASS="slide-title">$this</H3></TD></TR>
EOB
	# if we've retouched the pic, the exif summary may be in
	# $ORIGINAL_IMAGES
	exif1=`exists $ORIGINAL_IMAGES/$this.exif1 $this.exif1`
	if [ -s ${exif1:-/dev/null} ]; then
		# this.exif1 contains a oneline summary of EXIF data
		echo "<TR><TD COLSPAN=2 ALIGN="center"><SMALL>"
		divClass slide-exif $exif1
		echo "</SMALL><BR>&nbsp;</TD></TR>"
	fi >> $SLIDE
	if [ -s $this.txt ]; then
		echo "<TR><TD COLSPAN=2><P>"
		divClass slide-text $this.txt
		echo "<BR>&nbsp;</TD></TR>"
	fi >> $SLIDE
	echo "</TABLE></CENTER>" >> $SLIDE
	htm_footer slide "$this" $PIC >> $SLIDE
	install_if_new $slide
	DebugOff doPIC
}

doINDEX() {
	dir=`basename $curdir`
	DebugOn doINDEX $curdir $dir doINDEX:$curdir doINDEX:$dir
	prevdir=$1
	nextdir=$3
	shift 3
	index=$curdir/$INDEX_NAME
	INDEX=$index.$NewExt
	title=
	folderIcon=$FOLDER_ICON
	description=
        indexHead=$curdir/index-head.txt
        indexTail=$curdir/index-tail.txt
	get_properties meta.properties
	title=${title:-`get_title $curdir`}

	htm_header index "$CSS" "$title" > $INDEX
	cat << EOH >> $INDEX
<DIV CLASS="galleryheader">
<H2>$title</H2>
${description:+${CENTER_DESCRIPTION:+<CENTER>}<P>$description</P>${CENTER_DESCRIPTION:+</CENTER>}}
</DIV>
EOH
	if [ -s $indexHead ]; then
                divClass indexhead $indexHead
	fi >> $INDEX
	cat << EOH >> $INDEX
<CENTER>
<TABLE WIDTH="$INDEX_WIDTH" CELLPADDING="$INDEX_CELLPADDING" CELLSPACING="$INDEX_CELLSPACING" VALIGN="center">
<TBODY><TR>
EOH

	col=-1
	for d in $*
	do
		case "|$EXCLUDE_SUBDIRS|" in
		*\|$d\|*) continue;;
		esac
		[ -s $d/$INDEX_NAME ] || continue
		getIcon $d
		col=`expr \( $col + 1 \) % $COLS`
		case $col in
		0) echo "</TR><TR>" >> $INDEX;;
		esac
		getDIM $icon $index
		cat << EOD >> $INDEX
<TD CLASS="thumbnail" ALIGN="center" VALIGN="middle">
<A HREF="$d/$INDEX_NAME" CLASS="thumbnail">
<IMG SRC="$icon" WIDTH="$WIDTH" HEIGHT="$HEIGHT" BORDER="0"><BR>[&nbsp;${_folderLabel:-${_title:-$d}}&nbsp;]</A>
</TD>
EOD
	done >> $INDEX
	echo "</TR><TR></TR><TR>" >> $INDEX
	col=-1
	( sed -e 's,^./,,' -e '/-[twi]\./d' -e '/~$/d' -e '/^\./d' \
		-e '/index.h/d' -e '/info/d' \
		-e 's,\.[^\.]*$,,' -e '/^$/d' |
		sort -u; echo ) | (
		prev=
		read this

		while read next
		do
			THM=
			DebugOn $this doSLIDE
			$noSlides doSLIDE "${prev:-${prevdir:+../$prevdir}}" "$this" "${next:-${nextdir:+../$nextdir}}" "$CSS"
			col=`expr \( $col + 1 \) % $COLS`
			case $col in
			0) echo "</TR><TR>" >> $INDEX;;
			esac
			THM=${THM:-$this-t.jpg}
			if ! uptodate $this.ref $THM; then
				getDIM $THM $index
				cat << EOR > $this.ref
<A HREF="$this$HTML_EXT">
<IMG SRC="$THM" WIDTH="$WIDTH" HEIGHT="$HEIGHT" BORDER="0"><BR>$this</A>
EOR
			fi
			echo '<TD CLASS="thumbnail" ALIGN="center" VALIGN="middle">' >> $INDEX
			cat $this.ref >> $INDEX
			echo '</TD>' >> $INDEX
			prev=$this
			this=$next
			DebugOff $prev doSLIDE
		done
	)
	echo "</TR></TBODY></TABLE></CENTER>" >> $INDEX
	if [ -s $indexTail ]; then
                divClass indextail $indexTail
	fi >> $INDEX
	htm_footer index >> $INDEX
	$noIndex install_if_new $index
	
	DebugOff doINDEX $curdir $dir doINDEX:$curdir doINDEX:$dir
}

dodir() {
	DebugOn dodir dodir:$this dodir:$curdir $curdir $this
	cd $this
	[ -s $rc ] && . ./$rc
	curdir=`pwd`
	findRTOP
	# do dirs first
	dirs=`find . $findFollow -name . -o -prune -type d -print | sort |
	egrep -v "/($EXCLUDE_SUBDIRS)\$" | sed -e '/\/\./d' -e 's,^./,,'`
	$noSubdirs dodirs $dirs
	echo Processing $curdir...
	eval "find . -name . -o -prune \( -type f -o -type l \) \( $FIND_PICS \) -print" |
		doINDEX "$@" $dirs
	DebugOff dodir dodir:$this dodir:$curdir $curdir $this
}

dodirs() {
        DebugOn dodirs dodirs:$curdir
	(
		# PREV and NEXT may have been set on command line
		_prev=$PREV _next=$NEXT
		PREV= NEXT=
		prev= this= next=
		
		for next in $* ""
		do
			case "|$EXCLUDE_SUBDIRS|" in
			*\|$next\|*) continue;;
			esac
			if [ "$this" ]; then
				( dodir "$prev" "$this" "${next:-$_next}" )
			fi
			prev=${this:-$_prev}
			this=$next
		done
	)
        DebugOff dodirs dodirs:$curdir
}

dofiles() {
	for f in $*
	do
		[ -d $f ] && continue
		echo $f
	done | doINDEX "" "" ""
}

source_rc() {
        for d in $*
	do
                [ -s $d/$rc ] && . $d/$rc
	done
}

setBF() {
	case "$1" in
	no--no*) val=:;;
	do--do*) val=:;;
	*) val=;;
	esac
	var=`echo $1 | sed 's,--[dn]o,,'`
	eval $var=$val
}

DebugOn init

source_rc $HOME .
PREV=
NEXT=

while :
do
	case "$1" in
	*=*) eval "$1"; shift;;
	--) shift; break;;
	--findFollow) findFollow=-follow; shift;;
	--no[Ww]eb*) WEB_GEOMETRY=no; shift;;
	--skip[Dd]ir) exclude_subdir $2; shift 2;;
	--[dn]oThmBorder) setBF do$1 ; shift;;
	--[dn]o*) setBF no$1 ; shift;;
	--noUnsharp) UNSHARP=; shift;;
	--Unsharp) UNSHARP=$2; shift 2;;
	*) break;;
	esac
done

curdir=`pwd`

FIND_PICS=
pics=
for e in $PICS $RAW_PICS
do
        pics="$pics $e"
        e=`echo $e | toUpper`
        pics="$pics $e"
done
PICS="$pics"
for e in $PICS
do
	FIND_PICS="${FIND_PICS:+$FIND_PICS -o} -name '*$e'"
done
CONVERT=${CONVERT:-convert}
IDENTIFY=${IDENTIFY:-identify}
JPEG_QUALITY=${JPEG_QUALITY:-94}
WEB_GEOMETRY=${WEB_GEOMETRY:-760x700}
THM_GEOMETRY=${THM_GEOMETRY:-128}
CRW2EXT=${CRW2EXT:-jpg}
ORIGINAL_IMAGES=${ORIGINAL_IMAGES:-raw}
ROTATE_LIST=.rotate
ICONS=${ICONS:-/icons}
FOLDER_ICON=${FOLDER_ICON:-$ICONS/folder.gif}
HTML_EXT=${HTML_EXT:-.htm}
INDEX_NAME=${INDEX_NAME:-index$HTML_EXT}
INDEX_DESC="${INDEX_DESC:-Normal}"
STYLESHEET_DIR=${STYLESHEET_DIR:-res}
STYLESHEET=${STYLESHEET:-styles.css}
PRINT_STYLESHEET=${PRINT_STYLESHEET:-print.css}
COLS=${COLS:-7}
INDEX_CELLPADDING=${INDEX_CELLPADDING:-2}
INDEX_CELLSPACING=${INDEX_CELLSPACING:-2}
INDEX_WIDTH=${INDEX_WIDTH:-90%}
SLIDE_TEXT_WIDTH=${SLIDE_TEXT_WIDTH:-50%}

exclude_subdir $STYLESHEET_DIR
exclude_subdir $ORIGINAL_IMAGES
exclude_subdir $PRINT_IMAGES
exclude_subdir .junk

findTOP

case "$TOP" in
$curdir) ;;
*) [ "$rcfiles" ] && for f in $rcfiles; do . $f; done;;
esac
DebugOff init
case /$0 in
*/picalbum*)
	DebugOn main
	if [ -d ${1:-.} ]; then
		dodirs ${@:-.}
		case "${@:-.}" in
		.|*/*) ;;
		*)
			findRTOP
			DebugOn $curdir
			echo Processing $curdir...
			doINDEX "" "" "" $* < /dev/null
			DebugOff $curdir
			;;
		esac
	else
		dofiles $@
	fi
	DebugOff main
	;;
esac
